"""
MIT License

Copyright (c) 2024 sti1ltyping

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from CraftiGames.utils import packages # type: ignore
from CraftiGames.utils import config # type: ignore

default_skins = config.default_skins

class SkinTypes:
    """
    Wraps Skins
    ~~~~~~~~~~

    ==================================================================================================

    MIT License

    Copyright (c) 2024 sti1ltyping

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE."""
    
    def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
        self.player: str = player
        self.session: packages.aiohttp.ClientSession = session

        self.Default = self.Default(self.player, self.session)
        self.Marching = self.Marching(self.player, self.session)
        self.Walking = self.Walking(self.player, self.session)
        self.Crouching = self.Crouching(self.player, self.session)
        self.Crossed = self.Crossed(self.player, self.session)
        self.Criss_cross = self.Criss_cross(self.player, self.session)
        self.Ultimate = self.Ultimate(self.player, self.session)
        self.Isometric = self.Isometric(self.player, self.session)
        self.Head = self.Head(self.player, self.session)
        self.Custom = self.Custom(self.player, self.session)
        self.Cheering = self.Cheering(self.player, self.session)
        self.Relaxing = self.Relaxing(self.player, self.session)
        self.Trudging = self.Trudging(self.player, self.session)
        self.Cowering = self.Cowering(self.player, self.session)
        self.Pointing = self.Pointing(self.player, self.session)
        self.Lunging = self.Lunging(self.player, self.session)
        self.Dungeons = self.Dungeons(self.player, self.session)
        self.Facepalm = self.Facepalm(self.player, self.session)
        self.Sleeping = self.Sleeping(self.player, self.session)
        self.Dead = self.Dead(self.player, self.session)
        self.Archer = self.Archer(self.player, self.session)
        self.Kicking = self.Kicking(self.player, self.session)
        self.Mojavatar = self.Mojavatar(self.player, self.session)
        self.Reading = self.Reading(self.player, self.session)
        self.Bitzel = self.Bitzel(self.player, self.session)
        self.Pixel = self.Pixel(self.player, self.session)
        self.Skin = self.Skin(self.player, self.session)

    class Default:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Default bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/default/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Default full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/default/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Default face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/default/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Marching:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Marching bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/marching/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Marching full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/marching/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Marching face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/marching/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None


    class Walking:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Walking bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/walking/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Walking full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/walking/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Walking face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/walking/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None

    
    class Crouching:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Crouching bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/crouching/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Crouching full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/crouching/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Crouching face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/crouching/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Crossed:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Crossed bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/crossed/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Crossed full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/crossed/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Crossed face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/crossed/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Criss_cross:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Criss cross bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/criss_cross/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Criss cross full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/criss_cross/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Criss cross face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/criss_cross/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                
    
    class Ultimate:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Ultimate bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/ultimate/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Ultimate full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/ultimate/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Ultimate face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/ultimate/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Isometric:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Isometric bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/isometric/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Isometric full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/isometric/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Isometric face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/isometric/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                
        async def head(self) -> (packages.BytesIO | None):
            """
            Isometric head image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/isometric/{self.player}/head') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                
    
    class Head:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self
        

        async def full(self) -> (packages.BytesIO | None):
            """
            Head image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/head/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
                
    
    class Custom:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Custom bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/custom/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Custom full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/custom/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Custom face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/custom/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Cheering:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Cheering bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/cheering/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Cheering full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/cheering/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Cheering face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/cheering/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Relaxing:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Relaxing bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/relaxing/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Relaxing full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/relaxing/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Relaxing face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/relaxing/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                
    
    class Trudging:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Trudging bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/trudging/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Trudging full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/trudging/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Trudging face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/trudging/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Cowering:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Cowering bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/cowering/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Cowering full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/cowering/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Cowering face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/cowering/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Pointing:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Pointing bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/pointing/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Pointing full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/pointing/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Pointing face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/pointing/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Lunging:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Lunging bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/lunging/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Lunging full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/lunging/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Lunging face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/lunging/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                
    
    class Dungeons:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Dungeons bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/dungeons/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Dungeons full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/dungeons/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Dungeons face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/dungeons/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
        
    
    class Facepalm:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Facepalm bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/facepalm/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Facepalm full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/facepalm/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Facepalm face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/facepalm/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Sleeping:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Sleeping bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/sleeping/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Sleeping full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/sleeping/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
                

    class Dead:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Dead bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/dead/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Dead full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/dead/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Dead face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/dead/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Archer:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Archer bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/archer/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Archer full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/archer/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Archer face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/archer/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
        

    class Kicking:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Kicking bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/kicking/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Kicking full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/kicking/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Kicking face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/kicking/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Mojavatar:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Mojavatar bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/mojavatar/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Mojavatar full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/mojavatar/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
                

    class Reading:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Reading bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/reading/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Reading full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/reading/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Reading face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/reading/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Bitzel:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Bitzel bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/bitzel/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Bitzel full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/bitzel/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Bitzel face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/bitzel/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Pixel:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def bust(self) -> (packages.BytesIO | None):
            """
            Pixel bust image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/pixel/{self.player}/bust') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.bust()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.bust()
                
                else:
                    return None

        async def full(self) -> (packages.BytesIO | None):
            """
            Pixel full image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/pixel/{self.player}/full') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.full()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.full()
                
                else:
                    return None
        
        async def face(self) -> (packages.BytesIO | None):
            """
            Pixel face image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/pixel/{self.player}/face') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.face()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.face()
                
                else:
                    return None
                

    class Skin:
        def __init__(self, player: str, session: packages.aiohttp.ClientSession) -> None:
            self.player: str = player
            self.session: packages.aiohttp.ClientSession = session
            self.Retry = 0

        async def __cracked__(self):
            self.Retry += 1
            if self.Retry >= 5:
                self = None
            self.player = packages.random.choice(default_skins)
            return self

        async def default(self) -> (packages.BytesIO | None):
            """
            Skin of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/skin/{self.player}/default') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.default()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.default()
                
                else:
                    return None

        async def processed(self) -> (packages.BytesIO | None):
            """
            Processed image of the player.

            Returns:
            - Image Bytes of the skin of the player.
            - Image Bytes of the default skin as player is not premium.
            - None if something went wrong.
            """
            async with self.session.get(f'https://starlightskins.lunareclipse.studio/render/skin/{self.player}/processed') as response:

                if response.status == 200:
                    try:
                        self.Retry = 0
                        return packages.BytesIO(await response.read())
                    
                    except Exception:
                        self = await self.__cracked__()
                        return None if self is None else await self.processed()
                    
                elif response.status == 500:

                    self = await self.__cracked__()
                    return None if self is None else await self.processed()
                
                else:
                    return None


from setuptools import setup, find_packages

requirements = []
with open('requirements.txt') as requirements_file:
    requirements = requirements_file.read().splitlines()

with open('README.md') as README_file:
    description = README_file.read()

setup(
    name='CraftiGames.py',
    version='2.1.1',
    author='sti1ltyping',
    author_email='sti1ltyping.chillax@gmail.com',
    url='https://github.com/sti1ltyping/CraftiGames.py',
    packages=find_packages(),
    license='MIT',
    description='A Python based API wrapper for CraftiGames community',
    long_description=description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=requirements,
    python_requires='>=3.8.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
    ]
)


"""
MIT License

Copyright (c) 2024 sti1ltyping

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .Guilds import Guild, Clan
from CraftiGames.utils import packages, UnixTimestamp # type: ignore
from CraftiGames.Skins import SkinTypes # type: ignore

class PikaProfile:
    """
    Wrap Profile
    ~~~~~

    ==================================================================================================

    MIT License

    Copyright (c) 2024 sti1ltyping

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE."""

    def __init__(self, data: dict, session: packages.aiohttp.ClientSession):
        self.raw: dict = data
        self.session = session

    @property
    def skin(self) -> SkinTypes:
        return SkinTypes(self.username, self.session)
    
    @property
    def discord_verified(self) -> bool:
        """
        Returns:
        - `true` if player has connected with discord account.
        - `false` if player has not connected with discord account.
        """
        return self.raw.get("discord_verified", False)
    
    @property
    def discord_boosting(self) -> bool:
        """
        Returns:
        - `true` if player has boosted `PikaNetwork`'s official discord server.
        - `false` if player has not boosted `PikaNetwork`'s official discord server.
        """
        return self.raw.get("discord_boosting", False)
    
    @property
    def email_verified(self) -> bool:
        """
        Returns:
        - `true` if player has connected with email.
        - `false` if player has not connected with email.
        """
        return self.raw.get("email_verified", False)

    @property
    def username(self) -> str:
        """
        Returns:
        - username of the player.
        """
        return self.raw.get("username", '')
    
    @property
    def link(self) -> str:
        """
        Returns:
        - PikaNetwork link to the player's profile.
        """
        return f"https://stats.pika-network.net/player/{self.username}"
    
    @property
    def level(self) -> int:
        """
        Returns:
        - Level of the player.
        """
        return int(self.raw.get("rank", {}).get("level", 0))
    
    @property
    def level_percentage(self) -> float:
        """
        Returns:
        - level percentage of the player.
        """
        return float(self.raw.get("rank", {}).get("percentage", 0))
    
    @property
    def highest_minigame_rank(self) -> packages.Literal[
        "Developer", # 0
        "Admin",     # 1
        "Manager",   # 2
        "SrMod",     # 3
        "Moderator", # 4
        "Helper",    # 5
        "Trial",     # 6
        "Youtuber",  # 7
        "Champion",  # 8
        "Titan",     # 9
        "Elite",     # 10
        "VIP",       # 11
        "Unranked"   # 12 (Unranked)
        ]:
        """
        Returns:
        - Highest minigame rank of the player.
        """
        try:
            sorted_ranks = sorted(self.raw.get("ranks", []), key=lambda x: {
                "developer": 0, "admin": 1,"manager": 2, "srmod": 3, "moderator": 4, "helper": 5, "trial": 6, "youtuber": 7, "champion": 8, "titan": 9, "elite": 10, "vip": 11
                }.get(x.get('displayName', '').lower(), float('inf')))
            check = sorted_ranks[0]['displayName'] if sorted_ranks else 'Unranked'
            if check.lower() in ["developer", "admin", "manager", "srmod", "moderator", "helper", "trial", "youtuber", "champion", "titan", "elite", "vip"]:
                return check
            else:
                return 'Unranked'
        except Exception:
            return 'Unranked'
    
    @property
    def highest_practice_rank(self) -> packages.Literal[
        "Emerald", # 0
        "Diamond", # 1
        "Gold",    # 2
        "Silver",  # 3
        "Unranked" # 4
        ]:
        """
        Returns:
        - Highest practice rank of the player.
        """
        try:
            sorted_ranks = sorted(self.raw.get("ranks", []), key=lambda x: {
                "emerald": 0,"diamond": 1, "gold": 2, "silver": 3
            }.get(x.get('displayName', '').lower(), float('inf')))
            check = sorted_ranks[0]['displayName'] if sorted_ranks else 'Unranked'
            if check.lower() in ["emerald","diamond", "gold", "silver"]:
                return check
            else:
                return 'Unranked'
        except Exception:
            return 'Unranked'
        
    @property
    def last_seen(self) -> UnixTimestamp:
        """
        Returns:
        - UnixTimestamp (usable in discord timestamp) of player's last seen date-time.
        """
        last_seen_raw: int | None = self.raw.get("lastSeen", None)
        return int(packages.time.time()) if last_seen_raw == -1 or last_seen_raw is None else int(str(last_seen_raw)[:-3]) # Staffs have -1
    
    @property
    def last_seen_text(self) -> (str | None):
        """
        Returns:
        - last seen value in text format.
        """
        try:

            time_difference = packages.datetime.utcnow() - packages.datetime.utcfromtimestamp(self.last_seen)

            if time_difference.days > 0:
                return "1 day ago" if time_difference.days == 1 else f"{time_difference.days} days ago"
                    
            elif time_difference.seconds < 60:
                return "just now"
            
            elif time_difference.seconds < 3600:
                minutes = time_difference.seconds // 60
                return "1 min ago" if minutes == 1 else f"{minutes} mins ago"
            
            else:
                hours = time_difference.seconds // 3600
                return "1 hr ago" if hours == 1 else f"{hours} hrs ago"

        except Exception:
            return None
        
    @property
    def friend_list(self) -> list | None:
        """
        Returns:
        - List of player's friends. (iterable in ascending order)
        - None if the list is empty.
        """
        all_friends = self.raw.get('friends', [])
        friends = sorted([user['username'] for user in all_friends])

        return friends if friends else None
        
    @property
    def friend_count(self) -> int:
        """
        Returns:
        - Number of friends of the player.
        """
        frineds = self.friend_list
        return 0 if frineds is None else len(frineds)
    
    @property
    def is_staff(self) -> bool:
        """
        Returns:
        - `True` if player is part of the staff team.
        - `False` if player is not a staff.
        """
        return True if self.highest_minigame_rank.lower() in ["developer", "admin", "manager", "srmod", "moderator", "helper", "trial"] else False
    
    def guild(self) -> (Guild | None):
        """
        Extract guild from player's profile.
        
        Return:
        - Guild (class).
        - None if guild is not found.
        """
        clan = self.raw.get("clan", None)
        return None if clan is None else Guild(clan)
    

class JartexProfile:
    """
    Wrap Profile
    ~~~~~

    ==================================================================================================

    MIT License

    Copyright (c) 2024 sti1ltyping

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE."""

    def __init__(self, data: dict, session: packages.aiohttp.ClientSession):
        self.raw: dict = data
        self.session = session

    @property
    def skin(self) -> SkinTypes:
        return SkinTypes(self.username, self.session)
    
    @property
    def discord_verified(self) -> bool:
        """
        Returns:
        - `true` if player has connected with discord account.
        - `false` if player has not connected with discord account.
        """
        return self.raw.get("discord_verified", False)
    
    @property
    def discord_boosting(self) -> bool:
        """
        Returns:
        - `true` if player has boosted `JartexNetwork`'s official discord server.
        - `false` if player has not boosted `JartexNetwork`'s official discord server.
        """
        return self.raw.get("discord_boosting", False)
    
    @property
    def email_verified(self) -> bool:
        """
        Returns:
        - `true` if player has connected with email.
        - `false` if player has not connected with email.
        """
        return self.raw.get("email_verified", False)

    @property
    def username(self) -> str:
        """
        Returns:
        - username of the player.
        """
        return self.raw.get("username", '')
    
    @property
    def link(self) -> str:
        """
        Returns:
        - JartexNetwork link to the player's profile.
        """
        return f"https://stats.jartexnetwork.com/player/{self.username}"
    
    @property
    def level(self) -> int:
        """
        Returns:
        - Level of the player.
        """
        return int(self.raw.get("rank", {}).get("level", 0))
    
    @property
    def level_percentage(self) -> float:
        """
        Returns:
        - level percentage of the player.
        """
        return float(self.raw.get("rank", {}).get("percentage", 0))
    
    @property
    def rank(self) -> packages.Literal[
        "Owner",     # 0
        "Developer", # 1
        "Manager",   # 3
        "SrMod",     # 4
        "Moderator", # 5
        "JrMod",     # 6
        "Helper",    # 7
        "Trial",     # 8
        "Youtuber",  # 9
        "Crystal",   # 10
        "Diamond",   # 11
        "Gold",      # 12
        "Iron",      # 13
        "Unranked"   # 14 (Unranked)
        ]:
        """
        Returns:
        - Highest rank of the player.
        """
        try:
            sorted_ranks = sorted(self.raw.get("ranks", []), key=lambda x: {
                "owner": 0, "developer": 1, "manager": 2, "srmod": 3, "moderator": 5, "jrmod": 6, "helper": 7, "trial": 8, "youtuber": 9, "crystal": 10, "diamond": 11, "gold": 12, "iron": 13
                }.get(x.get('displayName', '').lower(), float('inf')))
            check = sorted_ranks[0]['displayName'] if sorted_ranks else 'Unranked'
            if check.lower() in ["owner", "developer", "manager", "srmod", "moderator", "jrmod", "helper", "trial", "youtuber", "crystal", "diamond", "gold", "iron"]:
                return check
            else:
                return 'Unranked'
        except Exception:
            return 'Unranked'
    
    @property
    def last_seen(self) -> UnixTimestamp:
        """
        Returns:
        - UnixTimestamp (usable in discord timestamp) of player's last seen date-time.
        """
        last_seen_raw: int | None = self.raw.get("lastSeen", None)
        return int(packages.time.time()) if last_seen_raw == -1 or last_seen_raw is None else int(str(last_seen_raw)[:-3]) # Staffs have -1
    
    @property
    def last_seen_text(self) -> (str | None):
        """
        Returns:
        - last seen value in text format.
        """
        try:

            time_difference = packages.datetime.utcnow() - packages.datetime.utcfromtimestamp(self.last_seen)

            if time_difference.days > 0:
                return "1 day ago" if time_difference.days == 1 else f"{time_difference.days} days ago"
                    
            elif time_difference.seconds < 60:
                return "just now"
            
            elif time_difference.seconds < 3600:
                minutes = time_difference.seconds // 60
                return "1 min ago" if minutes == 1 else f"{minutes} mins ago"
            
            else:
                hours = time_difference.seconds // 3600
                return "1 hr ago" if hours == 1 else f"{hours} hrs ago"

        except Exception:
            return None
    
    @property
    def friend_list(self) -> list | None:
        """
        Returns:
        - List of player's friends. (iterable in ascending order)
        - None if the list is empty.
        """
        all_friends = self.raw.get('friends', [])
        friends = sorted([user['username'] for user in all_friends])

        return friends if friends else None
      
    @property
    def friend_count(self) -> int:
        """
        Returns:
        - Number of friends of the player.
        """
        frineds = self.friend_list
        return 0 if frineds is None else len(frineds)
    
    @property
    def is_staff(self) -> bool:
        """
        Returns:
        - `True` if player is part of the staff team.
        - `False` if player is not a staff.
        """
        return True if self.rank.lower() in [
        "owner", "developer", "manager", "srmod", "moderator", "jrmod", "helper", "trial"
        ] else False
    
    def clan(self) -> (Clan | None):
        """
        Extract clan from player's profile.
        
        Return:
        - Clan (class).
        - None if clan is not found.
        """
        clan = self.raw.get("clan", None)
        return None if clan is None else Clan(clan)


"""
MIT License

Copyright (c) 2024 sti1ltyping

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from CraftiGames.Games import Stats # type: ignore

class Bedwars:
    """
    Wraps Bedwars 
    ~~~~~

    ==================================================================================================

    MIT License

    Copyright (c) 2024 sti1ltyping

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE."""
    
    def __init__(self, data: dict):
        self.raw: dict = data
        self.__stats__: Stats = Stats(self.raw)

    @property
    def wins(self) -> int:
        """
        Returns
        - Number of wins. (int)
        """
        return self.__stats__._wins(
            value=True,
            leaderboard=False
        )

    @property
    def wins_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of wins. (int)
        """
        return self.__stats__._wins(
            value=False,
            leaderboard=True
        )

    @property
    def losses(self) -> int:
        """
        Returns
        - Number of losses. (int)
        """
        return self.__stats__._losses(
            value=True,
            leaderboard=False
        )

    @property
    def losses_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of losses. (int)
        """
        return self.__stats__._losses(
            value=False,
            leaderboard=True
        )

    @property
    def wlr(self) -> float:
        """
        Returns
        - Wins is to losses ratio. (float)
        """
        return self.__stats__._wlr()

    @property
    def win_rate(self) -> float:
        """
        Returns
        - Win rate, calculated using previous wins & losses. (float)
        """
        return self.__stats__._win_rate()

    @property
    def lose_rate(self) -> float:
        """
        Returns
        - Lose rate, calculated using previous wins & losses. (float)
        """
        return self.__stats__._lose_rate()

    @property
    def kills(self) -> int:
        """
        Returns
        - Number of kills. (int)
        """
        return self.__stats__._kills(
            value=True,
            leaderboard=False
        )

    @property
    def kills_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of kills. (int)
        """
        return self.__stats__._kills(
            value=False,
            leaderboard=True
        )

    @property
    def deaths(self) -> int:
        """
        Returns
        - Number of deaths. (int)
        """
        return self.__stats__._deaths(
            value=True,
            leaderboard=False
        )

    @property
    def deaths_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of deaths. (int)
        """
        return self.__stats__._deaths(
            value=False,
            leaderboard=True
        )

    @property
    def kdr(self) -> int:
        """
        Returns
        - Kill is to death ratio. (float)
        """
        return self.__stats__._kdr()

    @property
    def kill_rate(self) -> float:
        """
        Returns
        - kill rate, calculated using previous kills & deaths. (float)
        """
        return self.__stats__._kill_rate()

    @property
    def death_rate(self) -> float:
        """
        Returns
        - death rate, calculated using previous kills & deaths. (float)
        """
        return self.__stats__._death_rate()

    @property
    def final_kills(self) -> int:
        """
        Returns
        - Number of final kills. (int)
        """
        return self.__stats__._final_kills(
            value=True,
            leaderboard=False
        )

    @property
    def final_kills_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of final kills. (int)
        """
        return self.__stats__._final_kills(
            value=False,
            leaderboard=True
        )

    @property
    def final_deaths(self) -> int:
        """
        Returns
        - Number of final deaths. (int)
        """
        return self.__stats__._final_deaths(
            value=True,
            leaderboard=False
        )

    @property
    def final_deaths_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of final deaths. (int)
        """
        return self.__stats__._final_deaths(
            value=False,
            leaderboard=True
        )

    @property
    def fkdr(self) -> float:
        """
        Returns
        - Final kills is to final deaths ratio. (float)
        """
        return self.__stats__._fkdr()

    @property
    def final_kill_rate(self) -> float:
        """
        Returns
        - final kill rate, calculated using previous final kills & final deaths. (float)
        """
        return self.__stats__._final_kill_rate()

    @property
    def final_death_rate(self) -> float:
        """
        Returns
        - final death rate, calculated using previous final kills & final deaths. (float)
        """
        return self.__stats__._final_death_rate()

    @property
    def beds_destroyed(self) -> int:
        """
        Returns
        - Number of beds destroyed. (int)
        """
        return self.__stats__._beds_destroyed(
            value=True,
            leaderboard=False
        )
    
    @property
    def beds_destroyed_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of beds destroyed. (int)
        """
        return self.__stats__._beds_destroyed(
            value=False,
            leaderboard=True
        )

    @property
    def highest_winstreak_reached(self) -> int:
        """
        Returns
        - highest winsteak reached. (int)
        """
        return self.__stats__._highest_winstreak_reached(
            value=True,
            leaderboard=False
        )
    
    @property
    def highest_winstreak_reached_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on highest winstreak. (int)
        """
        return self.__stats__._highest_winstreak_reached(
            value=False,
            leaderboard=True
        )

    @property
    def bow_kills(self) -> int:
        """
        Returns
        - Number of bow kills. (int)
        """
        return self.__stats__._bow_kills(
            value=True,
            leaderboard=False
        )

    @property
    def bow_kills_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of bow kills. (int)
        """
        return self.__stats__._bow_kills(
            value=False,
            leaderboard=True
        )

    @property
    def arrows_shot(self) -> int:
        """
        Returns
        - Number of arrows shot. (int)
        """
        return self.__stats__._arrows_shot(
            value=True,
            leaderboard=False
        )

    @property
    def arrows_shot_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of arrows shot. (int)
        """
        return self.__stats__._arrows_shot(
            value=False,
            leaderboard=True
        )

    @property
    def arrows_hit(self) -> int:
        """
        Returns
        - Number of arrows hit. (int)
        """
        return self.__stats__._arrows_hit(
            value=True,
            leaderboard=False
        )

    @property
    def arrows_hit_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of arrows hit. (int)
        """
        return self.__stats__._arrows_hit(
            value=False,
            leaderboard=True
        )

    @property
    def ahr(self) -> float:
        """
        Returns
        - Arrow hit ratio. (float)
        """
        return self.__stats__._ahr()

    @property
    def arrow_hit_rate(self) -> float:
        """
        Returns
        - Arrow hit rate, calculated using previous arrows shot & arrows hit. (float)
        """
        return self.__stats__._arrow_hit_rate()

    @property
    def arrow_miss_rate(self) -> float:
        """
        Returns
        - Arrow miss rate, calculated using previous arrows shot & arrows hit. (float)
        """
        return self.__stats__._arrow_miss_rate()

    @property
    def melee_kills(self) -> int:
        """
        Returns
        - Number of melee kills. (int)
        """
        return self.__stats__._melee_kills(
            value=True,
            leaderboard=False
        )

    @property
    def melee_kills_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of melee kills. (int)
        """
        return self.__stats__._melee_kills(
            value=False,
            leaderboard=True
        )

    @property
    def games_played(self) -> int:
        """
        Returns
        - Number of games played. (int)
        """
        return self.__stats__._games_played(
            value=True,
            leaderboard=False
        )

    @property
    def games_played_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of games played. (int)
        """
        return self.__stats__._games_played(
            value=False,
            leaderboard=True
        )

    @property
    def void_kills(self) -> int:
        """
        Returns
        - Number of void kills. (int)
        """
        return self.__stats__._void_kills(
            value=True,
            leaderboard=False
        )

    @property
    def void_kills_lb(self) -> int:
        """
        Returns
        - Leaderboard position based on number of void kills. (int)
        """
        return self.__stats__._void_kills(
            value=False,
            leaderboard=True
        )

    @property
    def exploited(self) -> int:
        """
        Number of number of games left `/hub`.

        Returns:
        - Number of games left unplayed.
        """
        return self.__stats__._exploited()